#include <QDoubleValidator>
#include <QGridLayout>
#include <QMessageBox>
#include <QDebug>

#include "CalibrationEditDialog_DXC120.h"
#include "usbtenki.h"
#include "usbtenki_cmds.h"


#define FRESH_AIR_VALUE_MIN  400
#define FRESH_AIR_VALUE_MAX 2000


CalibrationEditDialog_DXC120::CalibrationEditDialog_DXC120(QWidget *parent, TenkiDevice *td, int channel_id) : QDialog(parent)
{
	QDoubleValidator *valueValidator;

	tenki_device = td;
	btn_close = new QPushButton(tr("Close"));

	connect(btn_close, SIGNAL(clicked()), this, SLOT(close()));
	setWindowTitle(tr("Calibrate channel"));

	QGridLayout *gridLayout = new QGridLayout();

	QLabel *freshAirLabel = new QLabel("Fresh air (ppm)");
	QString freshAirValue = "450";
	freshAirEdit = new QLineEdit(freshAirValue);
	valueValidator = new QDoubleValidator(this);
	valueValidator->setRange(FRESH_AIR_VALUE_MIN, FRESH_AIR_VALUE_MAX, 0);
	freshAirEdit->setValidator(valueValidator);
	btn_set_fresh_air = new QPushButton("Set fresh air");
	connect(btn_set_fresh_air, &QPushButton::clicked, this, &CalibrationEditDialog_DXC120::setFreshAir);

	gridLayout->addWidget(freshAirLabel, 0, 0);
	gridLayout->addWidget(freshAirEdit, 0, 1);
	gridLayout->addWidget(btn_set_fresh_air, 0, 2);

	cb_disable = new QCheckBox("Disable auto-calibration");
	int checked = tenki_device->getDxc120Asc();
	cb_disable->setChecked(!checked);

	connect(cb_disable, &QCheckBox::stateChanged, this, &CalibrationEditDialog_DXC120::autoCalibrationStateChanged);

	QHBoxLayout *lay_btns = new QHBoxLayout();
	lay_btns->addStretch();
	lay_btns->addWidget(btn_close);

	QVBoxLayout *lay_vert = new QVBoxLayout();
	lay_vert->addSpacing(10);
	lay_vert->addWidget(cb_disable);
	lay_vert->addSpacing(10);
	lay_vert->addStretch();
	lay_vert->addLayout(gridLayout);
	lay_vert->addLayout(lay_btns);

	setLayout(lay_vert);
}

CalibrationEditDialog_DXC120::~CalibrationEditDialog_DXC120()
{

}

void CalibrationEditDialog_DXC120::close()
{
	QDialog::close();
}

void CalibrationEditDialog_DXC120::autoCalibrationStateChanged(int state)
{
	// Checked:   auto-calibration disabled
	// Unchecked: auto-calibration enabled

	int enabled = (state == Qt::Unchecked) ? 1 : 0;
	int res = tenki_device->setDxc120Asc(enabled);

}

void CalibrationEditDialog_DXC120::setFreshAir()
{
	if (freshAirEdit->hasAcceptableInput()) {
		QString value = freshAirEdit->text();
		int res = tenki_device->setDxc120Frc(value.toInt());
	}
	else {
		QMessageBox msgBox;
		QString message;
		message.sprintf("Value must be between %d and %d ppm", FRESH_AIR_VALUE_MIN, FRESH_AIR_VALUE_MAX);
		msgBox.setIcon(QMessageBox::Information);
		msgBox.setText(message);
		msgBox.exec();
	}
}
