#include <QDoubleValidator>
#include <QGridLayout>
#include <QMessageBox>
#include <QDebug>

#include "CalibrationEditDialog_DXC220.h"
#include "usbtenki.h"
#include "usbtenki_cmds.h"


#define CONCENTRATION_MIN 0
#define CONCENTRATION_MAX 200000


CalibrationEditDialog_DXC220::CalibrationEditDialog_DXC220(QWidget *parent, TenkiDevice *td, int channel_id) : QDialog(parent)
{
    USBTenki_channel *chn;
    QDoubleValidator *valueValidator;
    int pref_unit;

    tenki_device = td;

    chn = tenki_device->getChannelData(channel_id);

    btn_ok = new QPushButton(tr("OK"));
    btn_ok->setDefault(true);
    btn_cancel = new QPushButton(tr("Cancel"));

    connect(btn_ok, SIGNAL(clicked()), this, SLOT(applyClose()));
    connect(btn_cancel, SIGNAL(clicked()), this, SLOT(close()));
    setWindowTitle(tr("Calibrate channel"));

    QGridLayout *gridLayout = new QGridLayout();

    QLabel *label = new QLabel("Concentration (ppm)");
    concentrationEdit = new QLineEdit();
    valueValidator = new QDoubleValidator(this);
    valueValidator->setRange(CONCENTRATION_MIN, CONCENTRATION_MAX, 0);
    concentrationEdit->setValidator(valueValidator);
    gridLayout->addWidget(label, 0, 0);
    gridLayout->addWidget(concentrationEdit, 0, 1);

    QHBoxLayout *lay_btns = new QHBoxLayout();
    lay_btns->addStretch();
    lay_btns->addWidget(btn_cancel);
    lay_btns->addWidget(btn_ok);

    QVBoxLayout *lay_vert = new QVBoxLayout();

    lay_vert->addSpacing(10);
    lay_vert->addSpacing(10);
    lay_vert->addStretch();
    lay_vert->addLayout(gridLayout);
    lay_vert->addLayout(lay_btns);

    setLayout(lay_vert);
}

CalibrationEditDialog_DXC220::~CalibrationEditDialog_DXC220()
{

}

void CalibrationEditDialog_DXC220::applyClose()
{
    int i, res;

    if (concentrationEdit->hasAcceptableInput())
    {
        QString value = concentrationEdit->text();
        int res = tenki_device->setDxc220Kz(value.toInt());
        qDebug() << "Concentration set to " << value.toInt() << "; error=" << res;
    }
    else
    {
		QMessageBox msgBox;
		QString message;
		message.sprintf("Value must be between %d and %d ppm", CONCENTRATION_MIN, CONCENTRATION_MAX);
		msgBox.setIcon(QMessageBox::Information);
		msgBox.setText(message);
		msgBox.exec();
    }
}

void CalibrationEditDialog_DXC220::close()
{
    QDialog::close();
}
